{% extends "webpage/base.html" %}
{% load static %}
{% load webpage_extras %}
{% block title %} About {% endblock %}
{% block content %}
<div class="container">
    <h1>{% projects_metadata 'title' %}</h1>
    <p>
        {% projects_metadata 'description' %}
    </p>
</div>
{% endblock %}
