{% extends "webpage/base.html" %}
{% block content %}
<div id="modal-reg-closed" class="modal fade">
  <div class="modal-dialog">
    <div class="modal-content card-warning">
      <div class="modal-header">
        <button type="button" class="close" data-dismiss="modal" aria-hidden="true">×</button>
        <h4 class="modal-title">Registration is closed</h4>
      </div>
      <div class="modal-body">
        <p>Registration is closed. In order to ge an user account please email to <a href="mailto:matthias.schloegl@oeaw.ac.at">Matthias Schlögl</a> </p>
      </div>
      <div class="modal-footer">
        <button type="button" class="btn btn-default btn-primary" data-dismiss="modal">Close</button>
      </div>
    </div><!-- /.modal-content -->
  </div><!-- /.modal-dialog -->
</div><!-- /.modal -->
    <script type="text/javascript">
            $(document).ready(function() {
                $("#modal-reg-closed").modal();
            });
    </script>
{% endblock %}