# Generated by Django 2.2.12 on 2020-05-29 14:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('apis_metainfo', '0003_tempentityclass_published'),
    ]

    operations = [
        migrations.AddField(
            model_name='collection',
            name='published',
            field=models.BooleanField(default=False),
        ),
    ]
