# Generated by Django 2.2.13 on 2020-07-22 12:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('apis_metainfo', '0005_text_lang'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='text',
            name='lang',
        ),
    ]
