{% extends "webpage/base.html" %}
{% load static %}
{% load webpage_extras %}
{% block title %}{{ object.name }}{% endblock %}
{% block content %}
    <div class="container">
        <div class="card">
            <div class="card-header" style="text-align: center">
                <h1>Team</h1>
            </div>
            <div class="card-body">
                {% if objects %}
                    {% for object in objects %}
                        <div>
                            <h2>{{ object.name}}</h2>
                            <h3>{{ object.role }}</h3>
                            <p>
                                {{ object.description }}
                            </p>
                            <p>
                                <a href="{{ object.website}}">{{ object.website }}</a>
                            </p>
                        </div>
                    {% endfor %}
                {% else %}
                    <p>
                        No TeamMemeber objects have been created yet.
                        {% if user.is_authenticated %}
                            <strong>
                                go <a href="{% url 'infos:teammember_browse' %}">create</a> some
                            </strong>
                        {% endif %}
                    </p>
                {% endif %}
            </div>
            <div class="card-footer" style="text-align: center">
                <p>
                    {% if user.is_authenticated %}
                        <br />
                        <small>Data is fetch from <a href="{% url 'infos:teammember_browse' %}">TeamMember</a> object</small>
                    {% endif %}
                </p>
            </div>

        </div>
    </div>
{% endblock %}
