{% extends "browsing/create_base_template.html" %}
{% block title %}
    Create/Edit {{ class_name }}
{% endblock %}

{% block objectType %}
    <h1>Create/Edit
        {% if docstring %}
            <abbr title="{{ docstring}}">{{ class_name }}</abbr>
        {% else %}
            {{ class_name }}
        {% endif %}
    </h1>
{% endblock %}
