# Generated by Django 2.1.2 on 2020-02-16 11:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('apis_entities', '0002_auto_20200121_1227'),
    ]

    operations = [
        migrations.AlterField(
            model_name='person',
            name='gender',
            field=models.CharField(blank=True, choices=[('female', 'female'), ('male', 'male'), ('third gender', 'third gender')], max_length=15),
        ),
    ]
