# generated by appcreator
import django_filters
from django import forms
from . models import AboutTheProject, TeamMember, ProjectInst


class TeamMemberListFilter(django_filters.FilterSet):

    class Meta:
        model = TeamMember
        fields = "__all__"


class AboutTheProjectListFilter(django_filters.FilterSet):

    class Meta:
        model = AboutTheProject
        fields = "__all__"

class ProjectInstListFilter(django_filters.FilterSet):

    class Meta:
        model = ProjectInst
        fields = "__all__"
