{% extends "apis_entities/generic_list.html" %}
{% block Titel %}See all Works{% endblock %}
{% block list_title %}All Works{% endblock %}
{% block create_button %}
        <div>
            <a class="btn btn-primary btn-lg" href="{% url 'apis:apis_entities:work_create' %}">create new Work</a>
        </div>
{% endblock %}