# Generated by Django 2.1.12 on 2020-01-21 12:27

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('apis_vocabularies', '0001_initial'),
        ('apis_labels', '0001_initial'),
        ('apis_metainfo', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='label',
            name='label_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='apis_vocabularies.LabelType'),
        ),
        migrations.AddField(
            model_name='label',
            name='temp_entity',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='apis_metainfo.TempEntityClass'),
        ),
    ]
