<button id="tempEntity_{{ record.pk }}" type="button" class="btn btn-default reldelete" aria-label="Left Align" onclick=DeleteTempEntity('{{ record.pk }}','institutionEvent')><span class="glyphicon glyphicon-remove" aria-hidden="true"></span></button>