# Generated by Django 2.2.12 on 2020-05-29 14:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('apis_metainfo', '0002_auto_20200121_1227'),
    ]

    operations = [
        migrations.AddField(
            model_name='tempentityclass',
            name='published',
            field=models.BooleanField(default=False),
        ),
    ]
